package com.namasoft.common.utils.translation;

import java.text.MessageFormat;
import java.util.Map;
import java.util.Map.Entry;
import java.util.regex.Pattern;

import com.namasoft.common.constants.Language;
import com.namasoft.common.utilities.NaMaLogger;
import com.namasoft.common.utilities.ObjectChecker;

public abstract class MessageTranslatorBase implements MessageTranslator
{

	@Override
	public String translate(Language lang, String message, Object... arguments)
	{
		String translatedMessage = translateMessage(lang, message);
		if (translatedMessage == null)
			return null;
		return MessageFormat.format(translatedMessage == null ? message : translatedMessage, arguments);
	}

	public void processLang(Map<String, String> map, Language lang)
	{
		for (Entry<String, String> entry : map.entrySet())
		{
			processReference(entry, lang);
			processConcatenation(entry, lang);
		}
	}

	private void processReference(Entry<String, String> entry, Language lang)
	{
		if (entry.getValue().startsWith("@"))
		{
			String translatedValue = "";
			String[] parts = entry.getValue().split(Pattern.quote("@"));
			for (String part : parts)
			{
				if (ObjectChecker.isNotEmptyOrNull(part))
				{
					String translatedPart = translate(lang, part);
					if (translatedPart == null)
					{
						NaMaLogger.error("you did not define value for key {0}", part);
						translatedValue += part;
					}
					else
					{
						translatedValue += " " + translatedPart;
					}
				}
			}
			entry.setValue(translatedValue);
		}
	}

	private void processConcatenation(Entry<String, String> entry, Language lang)
	{
		if (entry == null)
			return;
		if (entry.getValue() == null)
		{
			NaMaLogger.error("you did not define value for key {0}", entry.getKey());
			return;
		}
		if (entry.getValue().startsWith("#"))
		{
			try
			{
				int part2Index = entry.getValue().indexOf("@");
				if (part2Index < 0)
					return;
				String part1 = entry.getValue().substring(1, part2Index);
				String part2 = entry.getValue().substring(part2Index + 1);
				entry.setValue(translate(lang, part1) + "|" + translate(lang, part2));
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
		}
	}

	public abstract String translateMessage(Language lang, String key);

}
